#!/usr/bin/env python2

'''
VolgaCTF 2015 Quals
lcg challenge

Exploit by Rogdham
2015.05.03
See http://r.rogdham.net/25
Licence: CC0

    This program is free software. It comes without any warranty, to the extent
    permitted by applicable law. You can redistribute it and/or modify it under
    the terms of the CC0 1.0 Universal Licence (French).
    See CC0 https://creativecommons.org/publicdomain/zero/1.0/deed.fr for more
    details.
'''

from lcg import LCG
import struct


magic = '\x89\x50\x4e\x47\x0d\x0a\x1a\x0a'


class LCG_decrypt(LCG):
    def __init__(self, data):
        data = data[:6]
        xored = ''.join(chr(a ^ b) for a, b in zip(map(ord, data),
                                                   map(ord, magic)))
        g0, g1, g2 = struct.unpack('<3H', xored)
        '''
        g0 = s
        g1 = a * g0 + b
        g2 = a * g1 + b
        g1 - g2 = a * (g0 - g1)
        '''
        self.m = 65521
        self.state = g0
        mb = 65536
        d12, d01 = g1 - g2 + self.m, g0 - g1 + self.m
        for _ in xrange((mb * mb + mb + self.m) / self.m):
            if d12 % d01 == 0:
                self.a = d12 / d01
                break
            d12 += self.m
        else:
            raise ValueError()
        self.b = (self.m + g1 - self.a * g0) % self.m

    def round(self):
        s = self.state
        LCG.round(self)
        return s

    def decrypt(self, data):
        gamma = self.generate_gamma(len(data))
        return ''.join([chr(d ^ g)  for d,g in zip(map(ord, data),
                                                   map(ord, gamma))])


if __name__ == '__main__':
    with open('flag.png.bin', 'rb') as f:
        data = f.read()
    dec = LCG_decrypt(data).decrypt(data)
    with open('flag.png', 'wb') as f:
        f.write(dec)
